%	TrackAnglesStat XTension
%	
%	Copyright (c) 2016 Chris Barilla
%	Some code was modified from existing Xtensions developed by BitPlane
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%        <Item name="Track Angles Statistic" icon="Matlab" tooltip="Create a statistic giving the angles of a chosen collection of surface tracks">
%          <Command>MatlabXT::TrackAnglesStat(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>
% 
%
%	Description: 
%   TrackAnglesStat XTension finds the angles of each track in a collection of surfaces.
%	Four angles are calculated the XY, XZ, YZ, and XYZ angles. 
%	The values of these angles are stored as a statistic.
%
%

function TrackAnglesStat(aImarisApplicationID)

	% connect to Imaris interface
	if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
	  javaaddpath ImarisLib.jar
	  vImarisLib = ImarisLib;
	  if ischar(aImarisApplicationID)
		aImarisApplicationID = round(str2double(aImarisApplicationID));
	  end
	  imarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
	else
	  imarisApplication = aImarisApplicationID;
	end
	

	% prompt user for the surface collection to evaluate
	vObject = chooseObjects( imarisApplication );
	

	% makes a structure that holds the track ids and maps them to the surface ids associated with the track
	tracksStruct = getTrackIndicesStruct( vObject );


    % this gives the number of surfaces that are in tracks (incremented in following loop).
    nTSurfaces = 0;
    
    % Iterate through all the tracks in collection of surfaces
    % Gets the coordinates of each surface in track from which the angles are calculated
    % Some of the following code has been adapted from the BitPlane developed XTension "XTTrackPlotAngles"
	for n = 1:length(tracksStruct)
			
		indexes = tracksStruct(n).ids;
		
		nTSurfaces = nTSurfaces + length(indexes);
		
				
		[vCoords, vTimes] = GetObjectsCoordsTimes( ...
			vObject, indexes );
		vCoords = vCoords';
		
		vNumberOfObjects = numel(vTimes);

		% order eventual unordered data
		vTimeMin = min(vTimes);
		vTimeMax = max(vTimes);

		vObjectAtTime = zeros(1, vTimeMax - vTimeMin + 1);
		vObjectAtTime(vTimes - vTimeMin + 1) = 1:vNumberOfObjects; % [1,3,0,0,2]
		vObjectsIndex = vObjectAtTime(vObjectAtTime ~= 0); % [1,3,2]

		% computes the track edges
		vStepsSize = vCoords(:, vObjectsIndex(2:vNumberOfObjects)) - ...
		  vCoords(:, vObjectsIndex(1:vNumberOfObjects-1));

		% get the angle for each pair of contiguous edges
		vAngles = zeros(4, vNumberOfObjects-2);
		for vObjecti = 1:vNumberOfObjects-2
		  vVector1 = vStepsSize(:, vObjecti);
		  vVector2 = vStepsSize(:, vObjecti+1);

		  vAngles(:, vObjecti) = [GetAngle(vVector1, vVector2); ...
			GetAngle(vVector1(1:2), vVector2(1:2)); ...
			GetAngle(vVector1([1,3]), vVector2([1,3])); ...
			GetAngle(vVector1(2:3), vVector2(2:3))];
        end
        
        tracksStruct(n).XYZ = vAngles(1,:)*180/pi;
        tracksStruct(n).XY = vAngles(2,:)*180/pi;
        tracksStruct(n).XZ = vAngles(3,:)*180/pi;
        tracksStruct(n).YZ = vAngles(4,:)*180/pi;
        tracksStruct(n).times = vTimes;
     	
	end
	
	
	
	% adds the angle data into 
	AddStats( vObject, tracksStruct, nTSurfaces, vObject.GetName());
	disp( 'Stats have been added.' )
	
end
	
	
%---------------------------------------------------------%
% Helper Functions Below
%---------------------------------------------------------%

%
%	GetAngle function will calculate the angle of vector
% 	This code is from BitPlane XTension "XTTrackPlotAngles"
%
%		@param aVector1 - array with 2 elements, the 2D coordinates of point one of vector
%		@param aVector2 - array with 2 elements, the 2D coordinates of point two of vector
%

function aAngle = GetAngle(aVector1, aVector2)
    vNormProduct = norm(aVector1) * norm(aVector2);

    if vNormProduct == 0
      aAngle = 0;
    else
      aAngle = acos(dot(aVector1,aVector2)/vNormProduct);
    end
end

    
%---------------------------------------------------------%

%
%	getTrackIndicesStruct function creates a structure that contains the track IDs and their associated surface IDs
%	
%		@param vObject - The ISurfaces object to be parsed for track information
%

function tracksStruct = getTrackIndicesStruct( vObject )

	% gets track and surface id info from ISurface Object
    trackids = int32(vObject.GetTrackIds);
	trackedges = vObject.GetTrackEdges;
	tracks = [trackids trackedges];

	
    trackIDs = unique(trackids);
    tracksStruct(length(trackIDs)) = struct();
    
    % iterate through trackIDs getting the associated surface ids
    for i = 1:length(trackIDs)
        inds = tracks(:,1) == trackIDs(i);
        tracksi = tracks(inds,:);
        ids = nan( size(tracksi,1)*2, 1);
        for j = 1:size(tracksi,1)
            ids(j:j+1) = tracksi(j,2:3);
        end
        ids = ids(~isnan(ids));
        ids = sort(unique(ids));
        
        % add Surface IDs and Track ID to structure
        tracksStruct(i).TrackID = trackIDs(i);
        tracksStruct(i).ids = ids;
    end
   
end
    

%---------------------------------------------------------%

%
% 	GetObjectsCoordsTimes function finds the coordintes of the surface objects, which are specified by the surface indexes
% 	This function was adapted from the BitPlane developed XTension "XTTrackPlotAngles"
%	
%		@param vObject - ISurfaces object to be parsed for surface ids and ultimately center of mass corrdinates
% 		@param aSelectedIndices - array of surface indices to get coordinate info from
%		@param vSize - the number of surfaces in the ISurface container 
%

function [aCoords, aTimes] = GetObjectsCoordsTimes( ...
  vObject, aSelectionIndices )
  

	aCoords = [];
	aTimes = [];

    nIndexes = numel(aSelectionIndices);
    aCoords = zeros(nIndexes, 3);
    aTimes = zeros(nIndexes, 1);
    % read selection
    for vIndex = 1:nIndexes
        vSurface = aSelectionIndices(vIndex);
        aCoords(vIndex, :) = vObject.GetCenterOfMass(vSurface);
        aTimes(vIndex) = vObject.GetTimeIndex(vSurface);
    end
    
end


%---------------------------------------------------------%

%
%	AddStats function creates a new statisitc object for the Surface Collection
%	
%		@param surfaces - ISurface object to be parsed for name information
%		@param vAngles - the struct created in main function that contains track and angle information
%

function AddStats( vObject, tracksStruct, nTSurfaces, surfaceName )


	% extract the information within the tracksStruct into arrays
	ids = nan( nTSurfaces, 1 );
	times = nan( nTSurfaces, 1 );
	trackids = nan( nTSurfaces, 1 );
	angleXY = nan( nTSurfaces, 1 );
	angleXZ = nan( nTSurfaces, 1 );
	angleYZ = nan( nTSurfaces, 1 );
	angleXYZ = nan( nTSurfaces, 1 );
	index = 0;
	for i = 1:length(tracksStruct)
		for j = 1:length(tracksStruct(i).ids)	
			if j ~= 1 & j ~= length(tracksStruct(i).ids)
				index = index + 1;	
				angleXY(index) = tracksStruct(i).XY(j-1);
				angleXZ(index) = tracksStruct(i).XY(j-1);
				angleYZ(index) = tracksStruct(i).XY(j-1);
				angleXYZ(index) = tracksStruct(i).XY(j-1);
				ids(index) = tracksStruct(i).ids(j);
				trackids(index) = tracksStruct(i).TrackID(1);
				times(index) = tracksStruct(i).times(j);
			end
		end
	end
	
	% trim arrays
	ids = ids( ~isnan(ids) );
	times = times( ~isnan(times) );
	trackids = trackids( ~isnan(trackids) );
	angleXY = angleXY( ~isnan(angleXY) );
	angleXZ = angleXZ( ~isnan(angleXZ) );
	angleYZ = angleYZ( ~isnan(angleYZ) );
	angleXYZ = angleXYZ( ~isnan(angleXYZ) );
	
	statSize = length(ids);
	
	% Make a factors matrix with all the features to be added to statistic
	statFactors = cell(4, statSize);
	statFactors(1, :) = repmat({'Surface'}, [statSize 1]);
	statFactors(2, :) = repmat({''}, [statSize 1]);
	statFactors(3, :) = num2cell(times);
	statFactors(3, :) = cellfun(@num2str, statFactors(3, :), 'UniformOutput', 0);
	statFactors(4, :) = num2cell(trackids);
	statFactors(4, :) = cellfun(@num2str, statFactors(4, :), 'UniformOutput', 0);

    factorNames = {'Category', 'Collection', 'Time', 'Track_ID'};
    
    
    surfaceName = char(surfaceName);


	% Make names of the statistics
	statName = ['Angle XY ' surfaceName ' ']; % Trailing space is to get Imaris to behave and show the units.
	statNameXY = repmat({statName}, [statSize 1]); 
	statName = ['Angle XZ ' surfaceName ' '];
	statNameXZ = repmat({statName}, [statSize 1]);
	statName = ['Angle YZ ' surfaceName ' ']; 
	statNameYZ = repmat({statName}, [statSize 1]);
	statName = ['Angle XYZ ' surfaceName ' ']; 
	statNameXYZ = repmat({statName}, [statSize 1]); 
	
	% make array with the unit for the angles
	statUnits = repmat({'Degrees'}, [statSize 1]);


	% add the four stats
    vObject.AddStatistics(statNameXY, angleXY, statUnits, ...
                        statFactors, factorNames, ids)
    
    vObject.AddStatistics(statNameXZ, angleXZ, statUnits, ...
                        statFactors, factorNames, ids)
    
    vObject.AddStatistics(statNameYZ, angleYZ, statUnits, ...
                        statFactors, factorNames, ids)
    
    vObject.AddStatistics(statNameXYZ, angleXYZ, statUnits, ...
                        statFactors, factorNames, ids)
	
end


%---------------------------------------------------------%

%
%	chooseObjects function creates a GUI to request the surfaces collection to be returned
%	
%		@param imarisApplication - the IApplication object to be parsed for Surface collection
%

function vObject = chooseObjects( imarisApplication )

	vSurpassScene = imarisApplication.GetSurpassScene;
	nTSurfaces=0;
	surfSelected=0;

	for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
		vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
		if imarisApplication.GetFactory.IsSurfaces(vDataItem)
			nTSurfaces=nTSurfaces+1;
			vSurfaces{nTSurfaces} = imarisApplication.GetFactory.ToSurfaces(vDataItem);    
			SurfNames{nTSurfaces}=char(vSurfaces{nTSurfaces}.GetName);
		end
	end 

	choice = menu('Choose Surface:',SurfNames);
	vObject = vSurfaces{choice};   
	
end

	



